{
   muu TrueColor library for TMT Pascal - Virtual palette fire example
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

USES mArea,mFormat,mSurface,mConsole,mTarga,mc_VESA,mc_Wrap,Nova,memory,CRT,Error,HTimer;

//{$DEFINE debug_wrapper}  // enable this ONLY for debugging

CONST xsize = 320;
      ysize = 200;

    // our Surface and Console, clear them just for neatness
VAR testsurface:    Surface := NIL;
    testconsole:    Console := NIL;
    dimer:      Timer := NIL;
    frames:     dword := 0;

TYPE Pal8 = ARRAY[BYTE,0..2] OF byte;

    // set textmode
PROCEDURE set_textmode; code;
ASM
  mov  ax,  $03
  int  $10
  ret
END;


PROCEDURE flame_it(m_surface:Surface); assembler;
  VAR m_data:   pointer := m_surface^.Data;
ASM
  mov  edi,  m_data
  add  edi,  xsize*ysize
  mov  ecx,  (xsize*ysize)-1
  xor  ecx,  -1
  inc  ecx
  xor  ax,   ax

@@fire_loop:
  xor  bx,   bx
  mov  al,   [edi+ecx+1]
  add  bx,   ax
  mov  al,   [edi+ecx-1]
  add  bx,   ax
  mov  al,   [edi+ecx+(xsize*2)]
  add  bx,   ax
  mov  al,   [edi+ecx+(xsize)]
  add  bx,   ax
  shr  bx,   2

  mov  [edi+ecx], bl
  inc  ecx
  jnz  @@fire_loop
END;


        // -------------- START OF MAIN CODE ----------------
begin
    // We will use the VBEConsole to display the image, therefore we overload
    // the testconsole type when creating it (creating means allocating)
    // (if debug_wrapper is defined, wrap VBEConsole to wrapper
  {$IFNDEF debug_wrapper}
    new(VBEConsole(testconsole),Create);
  {$ELSE}
    new(TTYConsole(testconsole),Create);
    DECLARE VAR deb: Console;
    BEGIN
      new(VBEConsole(deb),Create);
      testconsole^.Wrap(deb);
    END;
  {$ENDIF}

    // read options into Console, which will be used instead of the default
    // values if specified (here we load options from "mtc.cfg")
  testconsole^.getOptions(mtc_option_file);

    // options are set, and we can safely proceed opening our Console. Here
    // it's called "Image example", which is a suiting name:)
  testconsole^.Open('Image example',xsize,ysize,Format(unknown));

    // check if an error occurred when opening, and exit with error if so
  IF (testconsole^.Error) THEN
    fail('Failed to create console');

  testsurface := Surface(xsize,ysize+10,Format(VPAL));

  DECLARE VAR f:    file;
              t:    Pal8;
              c:    dword;
  BEGIN
    assign(f,'data\burn.pal');
    reset(f,1);
    blockread(f,t,768);
    close(f);
    FOR c := 0 TO 255 DO
    BEGIN
     testsurface^.sFormat.Palette[c,VBlue] := t[c,2] shl 2;
     testsurface^.sFormat.Palette[c,VGreen] := t[c,1] shl 2;
     testsurface^.sFormat.Palette[c,VRed] := t[c,0] shl 2;
    END;
  END;

  new(dimer,Create);

    // clear keyboard buffer
  WHILE keypressed DO readkey;

  randomize;
  dimer^.Start;
  DECLARE VAR randx:    word := 0;
  REPEAT
    FOR randx := 0 TO (xsize-1) DO
      IF (random(4) = 1) THEN
        mem[ofs(testsurface^.Data^)+xsize*(ysize-1)+randx] := 255;

    flame_it(testsurface);
    testconsole^.Copy(0,3,testsurface,Area(0,0,xsize-1,ysize-1-3));
    testconsole^.Update;
    frames +:= 1;
  UNTIL (keypressed);
  dimer^.Stop;

    // free memory used by Surface
  dispose(testsurface,Close);

    // close Console and free that one, too
  testconsole^.Close;
  dispose(testconsole,Destroy);

    // set textmode, and exit
  set_textmode;

  writeln('Frames per second: ', frames/(dimer^.Elapsed/1000):0:3);writeln;

  writeln('muu TrueColor library for TMT Pascal - Fire example');
  writeln('Copyright (C) 1999 Tor Arne Gustad (anden@online.no)');
end.
