{
   muu TrueColor library for TMT Pascal - Image viewer example
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

USES mArea,mFormat,mSurface,mConsole,mTarga,mc_VESA,mc_Wrap,Nova,mouse2K,CRT,Error;

//{$DEFINE debug_wrapper}  // enable this ONLY for debugging

    // our Surface and Console, clear them just for neatness
VAR testsurface:    Surface := NIL;
    testconsole:    Console := NIL;
    mouse:          m2K;

    // set textmode
PROCEDURE set_textmode; code;
ASM
  mov  ax,  $03
  int  $10
  ret
END;

        // -------------- START OF MAIN CODE ----------------
begin

  IF (paramcount <> 1) THEN
  BEGIN
    writeln(' usage: view.exe <image.tga>');
    halt;
  END;

    // load image passed to program by user
  testsurface := loadTGA(paramstr(1));

    // We will use the VBEConsole to display the image, therefore we overload
    // the testconsole type when creating it (creating means allocating)
    // (if debug_wrapper is defined, wrap VBEConsole to wrapper
  {$IFNDEF debug_wrapper}
    new(VBEConsole(testconsole),Create);
  {$ELSE}
    new(TTYConsole(testconsole),Create);
    DECLARE VAR deb: Console;
    BEGIN
      new(VBEConsole(deb),Create);
      testconsole^.Wrap(deb);
    END;
  {$ENDIF}

    // read options into Console, which will be used instead of the default
    // values if specified (here we load options from "mtc.cfg")
  testconsole^.getOptions(mtc_option_file);

    // options are set, and we can safely proceed opening our Console. Here
    // it's called "Image example", which is a suiting name:)
  testconsole^.Open('Image example',0,0,Format(unknown));

    // check if an error occurred when opening, and exit with error if so
  IF (testconsole^.Error) THEN
    fail('Failed to create console');

    // set up mouse
  DECLARE VAR diffX,diffY:  longint := 0;
  BEGIN
    new(mouse,Enable);

    diffX := testsurface^.Width-testconsole^.Info.Width;
    diffY := testsurface^.Height-testconsole^.Info.Height;

    IF (diffX < 0) THEN
      diffX := 0;
    IF (diffY < 0) THEN
      diffY := 0;

    mouse^.Restrict(0,0,diffX*2,diffY*2,relative);
    mouse^.Relative(diffX,diffY);
    mouse^.Place(diffX,diffY);
  END;
    // finally, look at the loaded image by moving the mouse cursor
  REPEAT
    mouse^.Update;
    testconsole^.Copy(0,0,testsurface,Area(mouse^.xPos,mouse^.yPos,testsurface^.Width,testsurface^.Height));
      // we have it copied, let's update the Console if this is needed (show)
    testconsole^.Update;
  UNTIL (keypressed) OR (mouse^.Button(Any));

    // clear keyboard buffer
  WHILE keypressed DO readkey;

    // disable mouse
  dispose(mouse,Disable);

    // free memory used by Surface
  dispose(testsurface,Close);

    // close Console and free that one, too
  testconsole^.Close;
  dispose(testconsole,Destroy);

    // set textmode, and exit
  set_textmode;

  writeln('muu TrueColor library for TMT Pascal - Image viewer example');
  writeln('Copyright (C) 1999 Tor Arne Gustad (anden@online.no)');
end.
