{
   muu TrueColor library for TMT Pascal - Nova 15bpp converter routines
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

PROCEDURE Nx86_S15_D32(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRRRGG GGGBBBBB -> ________ RRRRR___ GGGGG___ BBBBB___

ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    xor  eax,  eax
    
    // ax = _RRRRRGG GGGBBBBB, bl = GGGBBBBB
    mov  ax,   [esi]
    mov  bl,   al

    // bl = BBBBB___
    shl  bl,   3

    // ax = RRRRR___ GGGGG___
    shr  ax,   2
    shl  ah,   3
    and  ax,   $F8F8

    // eax = ________ RRRRR___ GGGGG___ BBBBB___
    shl  eax,  8
    mov  al,   bl

    mov  [edi],  eax

    add  esi,  2
    add  edi,  4
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D24(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRRRGG GGGBBBBB -> RRRRR___ GGGGG___ BBBBB___

ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    xor  eax,  eax
    
    // ax = _RRRRRGG GGGBBBBB, bl = GGGBBBBB
    mov  ax,   [esi]
    mov  bl,   al

    // bl = BBBBB___
    shl  bl,   3

    // ax = RRRRR___ GGGGG___
    shr  ax,   2
    shl  ah,   3
    and  ax,   $F8F8

    mov  [edi],  bl
    mov  [edi+1],  ax

    add  esi,  2
    add  edi,  3
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D16(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRRRGG GGGBBBBB -> RRRRRGGG GG_BBBBB
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    // ax = RRRRRGGG GGBBBBB_, bl = GGGBBBBB
    mov  ax,  [esi]
    mov  bl,  al
    shl  ax,  1

    // bl = ___BBBBB
    and  bl,  $1F

    // ax = RRRRRGGG GG_BBBBB
    and  ax,  $FFC0
    or   al,  bl

    mov  [edi],  ax

    add  esi,  2
    add  edi,  2
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D8(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRrrGG gggBBBbb -> RRRGGBBB
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    // ax = _RRRRRGG GGGBBBBB, bl = _RRRRRGG
    mov  ax,  [esi]
    mov  bl,  ah

    // bl = RRR_____
    shl  bl,  1
    and  bl,  $E0

    // ax = ___GG___ _____BBB
    shl  ah,  3
    shr  al,  2
    and  ax,  $1807

    // bl = RRRGGBBB
    or   bl,  al
    or   bl,  ah

    mov  [edi],  bl

    add  esi,  2
    inc  edi
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D32_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRRRGG GGGBBBBB -> ________ BBBBB___ GGGGG___ RRRRR___

ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    xor  eax,  eax
    
    // ax = _RRRRRGG GGGBBBBB, bl = GGGBBBBB
    mov  ax,   [esi]
    mov  bl,   al

    // bl = BBBBB___
    shl  bl,   3

    // ax = RRRRR___ GGGGG___
    shr  ax,   2
    shl  ah,   3
    and  ax,   $F8F8

    // eax = ________ BBBBB___ GGGGG___ RRRRR___
    xchg ah,   bl
    shl  eax,  8
    mov  al,   bl

    mov  [edi],  eax

    add  esi,  2
    add  edi,  4
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D24_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRRRGG GGGBBBBB -> BBBBB___ GGGGG___ RRRRR___

ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    xor  eax,  eax
    
    // ax = _RRRRRGG GGGBBBBB, bl = GGGBBBBB
    mov  ax,   [esi]
    mov  bl,   al

    // bl = BBBBB___
    shl  bl,   3

    // ax = RRRRR___ GGGGG___
    shr  ax,   2
    shl  ah,   3
    and  ax,   $F8F8

    xchg ah,   bl
    mov  [edi],  bl
    mov  [edi+2],  ax

    add  esi,  2
    add  edi,  3
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D16_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRRRGG GGGBBBBB -> BBBBBGGG GGGRRRRR
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    mov  ax,  [esi]
    mov  bx,  ax
    
    // bx = ___RRRRR BBBBB___
    shl  bl,  3
    shr  bh,  2

    // ax = _____GGG GG______
    shl  ax,  1
    and  ax,  $7C0

    // ax = BBBBBGGG GG_RRRRR
    or   ah,  bl
    or   al,  bh

    mov  [edi],  ax

    add  esi,  2
    add  edi,  2
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D15_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRRRGG GGGBBBBB -> _BBBBBGG GGGRRRRR
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    mov  ax,  [esi]
    mov  bx,  ax
    
    // bx = ___RRRRR _BBBBB__
    shl  bl,  2
    xor  bl,  $7C
    shr  bh,  3

    // ax = ______GG GGG_____
    and  ax,  $3E0

    // ax = _BBBBBGG GGGRRRRR
    or   ah,  bl
    or   al,  bh

    mov  [edi],  ax

    add  esi,  2
    add  edi,  2
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D8_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRrrGG gggBBBbb -> BBBGGRRR
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    mov  ax,  [esi]
    mov  bl,  al

    // bl = BBB____
    shl  bl,  3
    and  bl,  $E0

    // ax = _____RRR ___GG___
    shl  al,  3
    shr  ah,  4
    and  ax,  $718

    // al = BBBGGRRR
    or   al,  ah
    or   al,  bl

    mov  [edi],  al

    add  esi,  2
    inc  edi
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S15_D8_GRAY(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // _RRRRRGG GGGBBBBB -> II______ (sort of..)
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    xor  eax,  eax
    mov  ax,   [esi]
    mov  bl,   al

    // bl = __BBBBBb
    shl  bl,   1
    and  bl,   $3F

    // ax = __RRRRRr GGGGGBBB
    shr  ax,   2
    shl  ah,   1
    
    // al = __GGGGGg
    shr  al,   2
    and  al,   $3E

    add  al,   ah
    add  al,   bl
    mov  [edi],  al

    add  esi,  2
    inc  edi

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

