{
   muu TrueColor library for TMT Pascal - Targa image loader unit
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.       
}

UNIT mTARGA;

INTERFACE
{$R-,I+,Q-}

USES mFormat,mSurface,Nova,Memory,Error;

{$I include\tga_hdr.pas}

// --------------------------------------------------------------------------

CONST Loader_default_Format = BGR8888;

    // "LoadTGA_open" allocates a Surface and stores the image inside it
FUNCTION LoadTGA_open(m_name:string;m_format:Format): Surface;
FUNCTION LoadTGA_default(m_name:string): Surface;  
  OVERLOAD LoadTGA = LoadTGA_open;
  OVERLOAD LoadTGA = LoadTGA_default;

IMPLEMENTATION

    // Targa RLE decoder
PROCEDURE TGAdecodeRLE(source,target:pointer;datasize,pixelsize:dword); assembler;
ASM
  mov  esi,  source
  mov  edi,  target
  mov  ebx,  edi

  mov  ecx,  datasize
  shr  ecx,  2
  xor  eax,  eax
  rep  stosd
  mov  edi,  ebx

@@main_loop:
  xor  ecx,  ecx
  mov  cl,   [esi]
  inc  esi
  or   cl,   cl
  js   @@rle_packet
  jmp  @@raw_packet

    // Run-length encoded data
@@rle_packet:
  and  cl,   127
  inc  cl

@rle_loop1:
  mov  edx,  pixelsize

@rle_loop2:
  mov  al,   [esi+edx-1]
  mov  [edi+edx-1],  al
  dec  edx
  jnz  @rle_loop2

  add  edi,  pixelsize
  dec  ecx
  jnz  @rle_loop1

  add  esi,  pixelsize
  jmp  @@next

    // RAW data
@@raw_packet:
  inc  ecx
  mov  eax,  pixelsize
  mul  cl
  mov  ecx,  eax
  rep  movsb
  jmp  @@next

@@next:
  mov  eax,  edi
  sub  eax,  ebx
  cmp  eax,  datasize
  jb   @@main_loop
END;


// --------------------------------------------------------------------------
FUNCTION LoadTGA_open;
VAR header:     TGAheaderT;
    palette:    TGAPaletteT;
    f:          file;
    i_pitch:    byte := 0;
    i_size:     dword := 0;
    temp:       pointer := NIL;
BEGIN
  assign(f,m_name);
  {$I-} reset(f,1); {$I+}

    // does the file exist?
  IF (IOResult <> 0) THEN
    fail('"'+m_name+'" not found.');

    // is the file too small?
  IF (fileSize(f) < sizeof(header)) THEN
  BEGIN
    result := NIL;
    exit;
  END;

  seek(f,0);
  blockread(f,header,sizeof(header));

    // check if image is valid
  IF ((header.Descriptor AND 16) <> 0) OR
     (header.Width > $2000) OR
     (header.Height > $2000) AND
     ((header.Depth <> 32) OR
      (header.Depth <> 24) OR
      (header.Depth <> 16) OR
      (header.Depth <> 15) OR
      (header.Depth <>  8)) THEN
    fail('Failed to identify "'+m_name+'" as a valid Targa image.');

    // Start loading the image
  i_pitch := (header.Depth+4) SHR 3;

    // load RAW image data, and decode if needed
  DECLARE VAR c_pitch:      byte := (header.ColorMap.BitsPerPixel+4) shr 3;
              skip,read:    dword := 0;
              rawdata:      pointer := NIL;
  BEGIN
    IF (header.ColorMap.Length > 0) THEN
    BEGIN
      seek(f,18+header.IDLength);
      blockread(f,palette,header.ColorMap.Length*c_pitch);
    END;

    skip := 18+header.ColorMap.Length*c_pitch+header.IDLength;
    read := filesize(f)-skip;

    seek(f,skip);
    getmem(rawdata,read);
    blockread(f,rawdata^,read);
    close(f);

    // determine image size, allocate Surface memory
    i_size := header.Width*header.Height;

    IF (header.Depth = 8) THEN
      DECLARE VAR m_vpal:   VPaletteT;
                  count:    byte;
      BEGIN
        IF (header.ColorMap.Length > 0) THEN
            // color palette
          FOR count := 0 TO (header.ColorMap.Length-1) DO
          BEGIN
            m_vpal[count,VRed] := palette[count,rchn];
            m_vpal[count,VGreen] := palette[count,gchn];
            m_vpal[count,VBlue] := palette[count,bchn];
            m_vpal[count,VAlpha] := 0;
            // gray palette
          END ELSE FOR count := 0 TO 255 DO
          BEGIN
            m_vpal[count,VRed] := count;
            m_vpal[count,VGreen] := count;
            m_vpal[count,VBlue] := count;
            m_vpal[count,VAlpha] := 0;
          END;

        result := Surface(header.Width,header.Height,Format(m_vpal));
      END ELSE
        result := Surface(header.Width,header.Height,Format(header.Depth));

    // decode if data is compressed
    CASE (header.resultType) OF
      1 : memcpy(rawdata,result^.Data,i_size*i_pitch);
      2 : memcpy(rawdata,result^.Data,i_size*i_pitch);
      3 : memcpy(rawdata,result^.Data,i_size*i_pitch);
      9 : TGAdecodeRLE(rawdata,result^.Data,i_size*i_pitch,i_pitch);
     10 : TGAdecodeRLE(rawdata,result^.Data,i_size*i_pitch,i_pitch);

      ELSE
        fail('"'+m_name+'" is an unsupported Targa image.');
    END;

    freemem(rawdata,read);
  END;

    // Targa fixup: the 16bpp Targa files are really 15bpp!
  IF (Format(result).Pixelformat = BGR565) THEN
    result^.sFormat := Format(BGR555);

    // if necessary, convert the format to the requested or default format
  IF (Format(result).PixelFormat <> m_format.PixelFormat) THEN
    DECLARE VAR cconv:    Surface := Surface(result,Format(m_format));
    BEGIN
      cconv^.Copy(0,0,result,Area(result));
      dispose(result,Close);
      result := cconv;
    END;
    
    // flip if image is upside down
  IF ((header.Descriptor AND 32) = 0) THEN
  BEGIN
    getmem(temp,result^.Size);
    memcpy(result^.Data,temp,result^.Size);

    DECLARE VAR line:  dword := 0;
    FOR line := 0 TO (result^.Height-1) DO
      memcpy(ptr(ofs(temp^)+line*result^.Width*Pitch(result)),
             ptr(ofs(result^.Data^)+(result^.Height-1-line)*result^.Width*Pitch(result)),
             result^.Width*Pitch(result));

    freemem(temp,result^.Size);
  END;
END;

FUNCTION LoadTGA_default;
  result := LoadTGA_open(m_name,Format(Loader_default_Format));


end.
